/**
 * testingey
 * Copyright(c) 2011 Jesus A. Domingo <jesus.domingo@gmail.com>
 * MIT Licensed
 */

const sys = require('sys');

function _pt(color, message) {
  return (
    '\033[' + color + 'm' +
    message +
    '\033[0m'
  );
}

function _profile(cb) {
  var start = new Date().getTime();
  cb.call(undefined);
  return (new Date().getTime() - start);
}

function Testingey(name) {
  
  this.name = name;
  
  this._tests = {};
  this._timings = [];
  this._setUp = null;
  this._tearDown = null;
}

Testingey.prototype._enforceCB = function (name, cb) {
  if ('function' !== typeof cb) {
    throw new Error('Testingey#' + name + ': expecting a function as an argument');
  }
};

Testingey.prototype.setUp = function (cb) {
  this._enforceCB('setUp', cb);
  this._setUp = cb;
};

Testingey.prototype.tearDown = function (cb) {
  this._enforceCB('tearDown', cb);
  this._tearDown = cb;
};

Testingey.prototype.spec = function (desc, cb) {
  
  this._enforceCB('spec', cb);
  
  if (desc in this._tests) {
    console.warn('Spec [' + desc + '] being overwritten with new test');
  }

  this._tests[desc] = cb;
};

Testingey.prototype.summary = function (data) {

  if (data.timings.length) {
    var duration = data.timings.reduce(function (p, c) {
      return p + c;
    });
    duration = duration/1000;
    console.log('Tests finished in ' + duration + ' second(s)');
  }
    
  console.log(
    data.total + ' tests, ' +
    (data.failed > 0
      ? _pt(31, data.failed + ' failed')
      : _pt(32, data.failed + ' failed')
    )
  );

};

Testingey.prototype.run = function () {

  var spec = test = null,
      failed = 0,
      total = Object.keys(this._tests).length,
      timings = [];

  if (typeof this._setUp == 'function') {
    console.log('Setting up...');
    this._setUp.call(undefined);
  }

  if (total > 0) {
  
    console.log('Running test suite "' + this.name + '"');

    for (spec in this._tests) {
      test = this._tests[spec];
      try {
        sys.print(spec, '... ');
        timings.push(_profile(test));
        console.log(_pt(32, 'PASSED'));
      } catch (err) {
        console.log(_pt(31, 'FAILED'));
        console.log(_pt(31, '-- ' + err));
        ++failed;
      }
    }

    if (typeof this._tearDown == 'function') {
      sys.print('Cleaning up...');
      this._tearDown.call(undefined);
    }

    this.summary({
      total: total,
      failed: failed,
      timings: timings
    });
  
  } else {
    console.log('No tests to run.');
  }
};

/**
 * Factory method
 */
exports.createSuite = function (name) {
  return new Testingey(name);
};
